package utils

import (
	"encoding/json"
	"os"
	"path"

	"github.com/beego/beego/v2/core/logs"
)

const (
	DefaultBinlogPath = "./etalogs/binlog"
	DefaultApiLogPath = "./etalogs/apilog"
)

var (
	BinLog *logs.BeeLogger
	ApiLog *logs.BeeLogger
)

func init() {
	initApiLog()
	initBinLog()
}

// 初始化apilog日志
func initApiLog() {
	apilogPath := ApiLogPath
	if apilogPath == "" {
		apilogPath = DefaultApiLogPath
	}
	apilogFile := ApiLogFile
	if apilogFile == "" {
		apilogFile = "apilog.log"
	}
	os.MkdirAll(apilogPath, os.ModePerm)
	logFileName := path.Join(apilogPath, apilogFile)
	ApiLog = logs.NewLogger(1000000)
	logConf := getDefaultLogConfig()
	logConf.FileName = logFileName

	b, _ := json.Marshal(logConf)
	ApiLog.SetLogger(logs.AdapterFile, string(b))
	ApiLog.EnableFuncCallDepth(true)
}

// 初始化binlog日志
func initBinLog() {
	binlogPath := BinLogPath
	if binlogPath == "" {
		binlogPath = DefaultBinlogPath
	}
	binlogFile := BinLogFile
	if binlogFile == "" {
		binlogFile = "binlog.log"
	}
	os.MkdirAll(binlogPath, os.ModePerm)
	logFileName := path.Join(binlogPath, binlogFile)
	BinLog = logs.NewLogger(1000000)
	logConf := getDefaultLogConfig()
	logConf.FileName = logFileName

	b, _ := json.Marshal(logConf)
	BinLog.SetLogger(logs.AdapterFile, string(b))
	BinLog.EnableFuncCallDepth(true)
}

type logConfig struct {
	FileName string `json:"filename" description:"保存的文件名"`
	MaxLines int    `json:"maxlines"  description:"每个文件保存的最大行数，默认值 1000000"`
	MaxSize  int    `json:"maxsize" description:"每个文件保存的最大尺寸，默认值是 1 << 28, //256 MB"`
	Daily    bool   `json:"daily" description:"是否按照每天 logrotate，默认是 true"`
	MaxDays  int    `json:"maxdays" description:"文件最多保存多少天，默认保存 7 天"`
	Rotate   bool   `json:"rotate" description:"是否开启 logrotate，默认是 true"`
	Level    int    `json:"level" description:"日志保存的时候的级别，默认是 Trace 级别"`
	Color    bool   `json:"color" description:"日志是否输出颜色"`
}

func getDefaultLogConfig() logConfig {
	return logConfig{
		FileName: "",
		MaxLines: 10000000,
		MaxSize:  1 << 28,
		Daily:    true,
		MaxDays:  LogMaxDays,
		Rotate:   true,
		Level:    logs.LevelTrace,
		//Perm:     "",
	}
}
